==> mutual_credit-5.0.1.tar.gz: mutual_credit/src/Entity/ViewBuilder/TransactionViewBuilder.php <==
<?php

namespace Drupal\mcapi\Entity\ViewBuilder;

use Drupal\mcapi\Entity\Transaction;
use Drupal\mcapi\TransactionOperations;
use Drupal\mcapi\Entity\Storage\WalletStorage;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Utility\Token;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityViewBuilder;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Render controller for transactions.
 */
class TransactionViewBuilder extends EntityViewBuilder {

  protected $settings;
  protected $routeMatch;
  protected $token;

  /**
   *
   * @param type $entity_type
   * @param type $entity_repository
   * @param type $language_manager
   * @param type $theme_registry
   * @param type $entity_display_repository
   * @param ConfigFactory $config_factory
   * @param CurrentRouteMatch $route_match
   * @param Token $token
   */
  public function __construct($entity_type, $entity_repository, $language_manager, $theme_registry, $entity_display_repository, ConfigFactory $config_factory, CurrentRouteMatch $route_match, Token $token) {
    parent::__construct($entity_type, $entity_repository, $language_manager, $theme_registry, $entity_display_repository);
    $this->settings = $config_factory->get('mcapi.settings');
    $this->routeMatch = $route_match;
    $this->token = $token;
  }

  /**
   * {@inheritdoc}
   *
   * @todo update with entity_type.manager when core interface changes
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity.repository'),
      $container->get('language_manager'),
      $container->get('theme.registry'),
      $container->get('entity_display.repository'),
      $container->get('config.factory'),
      $container->get('current_route_match'),
      $container->get('token')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getBuildDefaults(EntityInterface $entity, $view_mode) {
    $build = parent::getBuildDefaults($entity, $view_mode);
    switch ($view_mode) {
      case 'full':
        $build['#links'] = [];
        if ($this->routeMatch->getRouteName() == 'entity.mcapi_transaction.canonical') {
          $build['#links'] = $this->linkList($entity, FALSE);
        }
        $build['#watermark'] = '';
        break;

      case 'sentence_links':
        if ($entity->parent->target_id == 0) {
          $build['link_list'] = $this->linkList($entity, TRUE);
        }
        //continue
      case 'sentence':
        // This way of doing it we can't quite use the parent theme callback
        // mcapi_transaction, so we'll just add this div wrapper by hand.
        unset($build['#theme'], $build['#mcapi_transaction']);
        $build['#prefix'] = '<div class = "mcapi_transaction-sentence">';
        $build['#suffix'] = '</div>';
        $template = $this->settings->get('sentence_template');
        $build['#markup'] = $this->token->replace($template, ['xaction' => $entity]);
        break;
    }
    $build += [
      '#attributes' => [
        'class' => [
          'transaction',
          'type-' . $entity->type->target_id,
          'state-' . $entity->state->target_id,
          ($view_mode == 'full' ? 'certificate' : $view_mode)
        ]
      ],
      '#attached' => [
        // For some reason in Renderer::updatestack, this bubbles up twice.
        'library' => ['mcapi/mcapi.transaction'],
      ],
    ];
    // @todo we might need to use the post-render cache to get the links right instead of template_preprocess_mcapi_transaction
    return $build;
  }

  /**
   * Make a renderable array for transaction operations
   * @param Transaction $transaction
   * @param bool $include_view_op
   * @return array
   *   A renderable array
   */
  private function linkList(Transaction $transaction, $include_view_op = TRUE) : array {
    $renderable = [];
    if (!$transaction->isNew()) {
      return [
        '#lazy_builder' => [
          get_called_class() . '::renderLinks', [
            $transaction->serial->value,
            $include_view_op
          ]
        ]
      ];
    }
    return $renderable;
  }

  /**
   * @param int $serial
   * @param bool $include_view_op
   *
   * @todo Better cache management of the links.
   */
  public static function renderlinks(int $serial, $include_view_op = TRUE) : array {
    $output = [
      '#theme' => 'item_list',
      '#type' => 'ul',
      '#items' => [],
      '#attributes' => ['class' => ['transaction-operations']],
    ];
    $operations = TransactionOperations::get(
      Transaction::loadBySerial($serial),
      $include_view_op
    );
    foreach ($operations as $name => $op) {
      if ($name == 'delete') {
        $dest_route = 'entity.mcapi_transaction.collection';
      }
      else {
        $dest_route = '<current>';
      }
      $dest_url = Url::fromRoute($dest_route)->toString();
      $output['#items'][]= Link::fromTextAndUrl(
        $op['title'],
        $op['url']->setoption('query', ['destination' => $dest_url])
      );
    }
    return $output;
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    $callbacks = parent::trustedCallbacks();
    $callbacks[] = 'renderLinks';
    return $callbacks;
  }

}

==> mutual_credit-5.0.8.tar.gz: mutual_credit/src/Entity/ViewBuilder/TransactionViewBuilder.php <==
<?php

namespace Drupal\mcapi\Entity\ViewBuilder;

use Drupal\mcapi\Entity\Transaction;
use Drupal\mcapi\TransactionOperations;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Utility\Token;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityViewBuilder;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Render controller for transactions.
 */
class TransactionViewBuilder extends EntityViewBuilder {

  protected $settings;
  protected $routeMatch;
  protected $token;

  /**
   *
   * @param type $entity_type
   * @param type $entity_repository
   * @param type $language_manager
   * @param type $theme_registry
   * @param type $entity_display_repository
   * @param ConfigFactory $config_factory
   * @param CurrentRouteMatch $route_match
   * @param Token $token
   */
  public function __construct($entity_type, $entity_repository, $language_manager, $theme_registry, $entity_display_repository, ConfigFactory $config_factory, CurrentRouteMatch $route_match, Token $token) {
    parent::__construct($entity_type, $entity_repository, $language_manager, $theme_registry, $entity_display_repository);
    $this->settings = $config_factory->get('mcapi.settings');
    $this->routeMatch = $route_match;
    $this->token = $token;
  }

  /**
   * {@inheritdoc}
   *
   * @todo update with entity_type.manager when core interface changes
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity.repository'),
      $container->get('language_manager'),
      $container->get('theme.registry'),
      $container->get('entity_display.repository'),
      $container->get('config.factory'),
      $container->get('current_route_match'),
      $container->get('token')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getBuildDefaults(EntityInterface $entity, $view_mode) {
    $build = parent::getBuildDefaults($entity, $view_mode);
    switch ($view_mode) {
      case 'full':
        $build['#links'] = [];
        if ($this->routeMatch->getRouteName() == 'entity.mcapi_transaction.canonical') {
          $build['#links'] = $this->linkList($entity, FALSE);
        }
        $build['#watermark'] = '';
        break;

      case 'sentence_links':
        if ($entity->parent->target_id == 0) {
          $build['link_list'] = $this->linkList($entity, TRUE);
        }
        //continue
      case 'sentence':
        // This way of doing it we can't quite use the parent theme callback
        // mcapi_transaction, so we'll just add this div wrapper by hand.
        unset($build['#theme'], $build['#mcapi_transaction']);
        $build['#prefix'] = '<div class = "mcapi_transaction-sentence">';
        $build['#suffix'] = '</div>';
        $template = $this->settings->get('sentence_template');
        $sentence =$this->token->replace($template, ['xaction' => $entity]);
        // @todo replace this in 9.4 with RenderPlain
        $build['#markup'] = \Drupal\Component\Render\PlainTextOutput::renderFromHtml($sentence);
        break;
    }
    $build += [
      '#attributes' => [
        'class' => [
          'transaction',
          'type-' . $entity->type->target_id,
          'state-' . $entity->state->target_id,
          ($view_mode == 'full' ? 'certificate' : $view_mode)
        ]
      ],
      '#attached' => [
        // For some reason in Renderer::updatestack, this bubbles up twice.
        'library' => ['mcapi/mcapi.transaction'],
      ],
    ];
    // @todo we might need to use the post-render cache to get the links right instead of template_preprocess_mcapi_transaction
    return $build;
  }

  /**
   * Make a renderable array for transaction operations
   * @param Transaction $transaction
   * @param bool $include_view_op
   * @return array
   *   A renderable array
   */
  private function linkList(Transaction $transaction, $include_view_op = TRUE) : array {
    $renderable = [];
    if (!$transaction->isNew()) {
      return [
        '#lazy_builder' => [
          get_called_class() . '::renderLinks', [
            $transaction->serial->value,
            $include_view_op
          ]
        ]
      ];
    }
    return $renderable;
  }

  /**
   * @param int $serial
   * @param bool $include_view_op
   *
   * @todo Better cache management of the links.
   */
  public static function renderlinks(int $serial, $include_view_op = TRUE) : array {
    $output = [
      '#theme' => 'item_list',
      '#type' => 'ul',
      '#items' => [],
      '#attributes' => ['class' => ['transaction-operations']],
    ];
    $operations = TransactionOperations::get(
      Transaction::loadBySerial($serial),
      $include_view_op
    );
    foreach ($operations as $name => $op) {
      if ($name == 'delete') {
        $dest_route = 'entity.mcapi_transaction.collection';
      }
      else {
        $dest_route = '<current>';
      }
      $dest_url = Url::fromRoute($dest_route)->toString();
      $output['#items'][]= Link::fromTextAndUrl(
        $op['title'],
        $op['url']->setoption('query', ['destination' => $dest_url])
      );
    }
    return $output;
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    $callbacks = parent::trustedCallbacks();
    $callbacks[] = 'renderLinks';
    return $callbacks;
  }

}
